@@ -12,7 +12,6 @@
 """ Unit test for daemon module
 """
 
-import __builtin__
 import os
 import sys
 from StringIO import StringIO
@@ -22,6 +21,7 @@ import errno
 import signal
 import socket
 from types import ModuleType
+import atexit
 
 import scaffold
 from test_pidlockfile import (
@@ -45,7 +45,7 @@ class Exception_TestCase(scaffold.Exception_TestCase):
                 min_args = 1,
                 types = (Exception,),
             ),
-            daemon.daemon.DaemonEnvironmentOSError: dict(
+            daemon.daemon.DaemonOSEnvironmentError: dict(
                 min_args = 1,
                 types = (daemon.daemon.DaemonError, OSError),
             ),
@@ -61,8 +61,7 @@ class change_working_directory_TestCase(scaffold.TestCase):
 
     def setUp(self):
         """ Set up test fixtures """
-        self.mock_outfile = StringIO()
-        self.mock_tracker = scaffold.MockTracker(self.mock_outfile)
+        self.mock_tracker = scaffold.MockTracker()
 
         scaffold.mock(
             "os.chdir",
@@ -85,15 +84,14 @@ class change_working_directory_TestCase(scaffold.TestCase):
             Called os.chdir(%(directory)r)
             """ % vars()
         daemon.daemon.change_working_directory(**args)
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_raises_daemon_error_on_os_error(self):
         """ Should raise a DaemonError on receiving and OSError. """
         args = self.test_args
         test_error = OSError(errno.ENOENT, "No such directory")
         os.chdir.mock_raises = test_error
-        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        expect_error = daemon.daemon.DaemonOSEnvironmentError
         self.failUnlessRaises(
             expect_error,
             daemon.daemon.change_working_directory, **args)
@@ -103,7 +101,7 @@ class change_working_directory_TestCase(scaffold.TestCase):
         args = self.test_args
         test_error = OSError(errno.ENOENT, "No such directory")
         os.chdir.mock_raises = test_error
-        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        expect_error = daemon.daemon.DaemonOSEnvironmentError
         try:
             daemon.daemon.change_working_directory(**args)
         except expect_error, exc:
@@ -116,8 +114,7 @@ class change_root_directory_TestCase(scaffold.TestCase):
 
     def setUp(self):
         """ Set up test fixtures """
-        self.mock_outfile = StringIO()
-        self.mock_tracker = scaffold.MockTracker(self.mock_outfile)
+        self.mock_tracker = scaffold.MockTracker()
 
         scaffold.mock(
             "os.chdir",
@@ -144,8 +141,7 @@ class change_root_directory_TestCase(scaffold.TestCase):
             ...
             """ % vars()
         daemon.daemon.change_root_directory(**args)
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_changes_root_directory_to_specified_directory(self):
         """ Should change root directory to specified directory. """
@@ -156,15 +152,14 @@ class change_root_directory_TestCase(scaffold.TestCase):
             Called os.chroot(%(directory)r)
             """ % vars()
         daemon.daemon.change_root_directory(**args)
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_raises_daemon_error_on_os_error_from_chdir(self):
         """ Should raise a DaemonError on receiving an OSError from chdir. """
         args = self.test_args
         test_error = OSError(errno.ENOENT, "No such directory")
         os.chdir.mock_raises = test_error
-        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        expect_error = daemon.daemon.DaemonOSEnvironmentError
         self.failUnlessRaises(
             expect_error,
             daemon.daemon.change_root_directory, **args)
@@ -174,7 +169,7 @@ class change_root_directory_TestCase(scaffold.TestCase):
         args = self.test_args
         test_error = OSError(errno.EPERM, "No chroot for you!")
         os.chroot.mock_raises = test_error
-        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        expect_error = daemon.daemon.DaemonOSEnvironmentError
         self.failUnlessRaises(
             expect_error,
             daemon.daemon.change_root_directory, **args)
@@ -184,7 +179,7 @@ class change_root_directory_TestCase(scaffold.TestCase):
         args = self.test_args
         test_error = OSError(errno.ENOENT, "No such directory")
         os.chdir.mock_raises = test_error
-        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        expect_error = daemon.daemon.DaemonOSEnvironmentError
         try:
             daemon.daemon.change_root_directory(**args)
         except expect_error, exc:
@@ -197,8 +192,7 @@ class change_file_creation_mask_TestCase(scaffold.TestCase):
 
     def setUp(self):
         """ Set up test fixtures """
-        self.mock_outfile = StringIO()
-        self.mock_tracker = scaffold.MockTracker(self.mock_outfile)
+        self.mock_tracker = scaffold.MockTracker()
 
         scaffold.mock(
             "os.umask",
@@ -221,15 +215,14 @@ class change_file_creation_mask_TestCase(scaffold.TestCase):
             Called os.umask(%(mask)r)
             """ % vars()
         daemon.daemon.change_file_creation_mask(**args)
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_raises_daemon_error_on_os_error_from_chdir(self):
         """ Should raise a DaemonError on receiving an OSError from umask. """
         args = self.test_args
         test_error = OSError(errno.EINVAL, "Whatchoo talkin' 'bout?")
         os.umask.mock_raises = test_error
-        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        expect_error = daemon.daemon.DaemonOSEnvironmentError
         self.failUnlessRaises(
             expect_error,
             daemon.daemon.change_file_creation_mask, **args)
@@ -239,7 +232,7 @@ class change_file_creation_mask_TestCase(scaffold.TestCase):
         args = self.test_args
         test_error = OSError(errno.ENOENT, "No such directory")
         os.umask.mock_raises = test_error
-        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        expect_error = daemon.daemon.DaemonOSEnvironmentError
         try:
             daemon.daemon.change_file_creation_mask(**args)
         except expect_error, exc:
@@ -252,8 +245,7 @@ class change_process_owner_TestCase(scaffold.TestCase):
 
     def setUp(self):
         """ Set up test fixtures """
-        self.mock_outfile = StringIO()
-        self.mock_tracker = scaffold.MockTracker(self.mock_outfile)
+        self.mock_tracker = scaffold.MockTracker()
 
         scaffold.mock(
             "os.setuid",
@@ -287,8 +279,7 @@ class change_process_owner_TestCase(scaffold.TestCase):
             Called os.setuid(...)
             """ % vars()
         daemon.daemon.change_process_owner(**args)
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_changes_group_id_to_gid(self):
         """ Should change process GID to specified value. """
@@ -299,8 +290,7 @@ class change_process_owner_TestCase(scaffold.TestCase):
             ...
             """ % vars()
         daemon.daemon.change_process_owner(**args)
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_changes_user_id_to_uid(self):
         """ Should change process UID to specified value. """
@@ -311,15 +301,14 @@ class change_process_owner_TestCase(scaffold.TestCase):
             Called os.setuid(%(uid)r)
             """ % vars()
         daemon.daemon.change_process_owner(**args)
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_raises_daemon_error_on_os_error_from_setgid(self):
         """ Should raise a DaemonError on receiving an OSError from setgid. """
         args = self.test_args
         test_error = OSError(errno.EPERM, "No switching for you!")
         os.setgid.mock_raises = test_error
-        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        expect_error = daemon.daemon.DaemonOSEnvironmentError
         self.failUnlessRaises(
             expect_error,
             daemon.daemon.change_process_owner, **args)
@@ -329,7 +318,7 @@ class change_process_owner_TestCase(scaffold.TestCase):
         args = self.test_args
         test_error = OSError(errno.EPERM, "No switching for you!")
         os.setuid.mock_raises = test_error
-        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        expect_error = daemon.daemon.DaemonOSEnvironmentError
         self.failUnlessRaises(
             expect_error,
             daemon.daemon.change_process_owner, **args)
@@ -339,7 +328,7 @@ class change_process_owner_TestCase(scaffold.TestCase):
         args = self.test_args
         test_error = OSError(errno.EINVAL, "Whatchoo talkin' 'bout?")
         os.setuid.mock_raises = test_error
-        expect_error = daemon.daemon.DaemonEnvironmentOSError
+        expect_error = daemon.daemon.DaemonOSEnvironmentError
         try:
             daemon.daemon.change_process_owner(**args)
         except expect_error, exc:
@@ -348,12 +337,11 @@ class change_process_owner_TestCase(scaffold.TestCase):
 
 
 class prevent_core_dump_TestCase(scaffold.TestCase):
-    """ Test cases for prevent_core_dump function """
+    """ Test cases for prevent_core_dump function. """
 
     def setUp(self):
-        """ Set up test fixtures """
-        self.mock_outfile = StringIO()
-        self.mock_tracker = scaffold.MockTracker(self.mock_outfile)
+        """ Set up test fixtures. """
+        self.mock_tracker = scaffold.MockTracker()
 
         self.RLIMIT_CORE = object()
         scaffold.mock(
@@ -367,11 +355,11 @@ class prevent_core_dump_TestCase(scaffold.TestCase):
             tracker=self.mock_tracker)
 
     def tearDown(self):
-        """ Tear down test fixtures """
+        """ Tear down test fixtures. """
         scaffold.mock_restore()
 
     def test_sets_core_limit_to_zero(self):
-        """ Should set the RLIMIT_CORE resource to zero """
+        """ Should set the RLIMIT_CORE resource to zero. """
         expect_resource = self.RLIMIT_CORE
         expect_limit = (0, 0)
         expect_mock_output = """\
@@ -382,18 +370,17 @@ class prevent_core_dump_TestCase(scaffold.TestCase):
                 %(expect_limit)r)
             """ % vars()
         daemon.daemon.prevent_core_dump()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_raises_error_when_no_core_resource(self):
-        """ Should raise ValueError if no RLIMIT_CORE resource """
+        """ Should raise DaemonError if no RLIMIT_CORE resource. """
         def mock_getrlimit(res):
             if res == resource.RLIMIT_CORE:
                 raise ValueError("Bogus platform doesn't have RLIMIT_CORE")
             else:
                 return None
         resource.getrlimit.mock_returns_func = mock_getrlimit
-        expect_error = ValueError
+        expect_error = daemon.daemon.DaemonOSEnvironmentError
         self.failUnlessRaises(
             expect_error,
             daemon.daemon.prevent_core_dump)
@@ -404,8 +391,7 @@ class close_file_descriptor_if_open_TestCase(scaffold.TestCase):
 
     def setUp(self):
         """ Set up test fixtures """
-        self.mock_outfile = StringIO()
-        self.mock_tracker = scaffold.MockTracker(self.mock_outfile)
+        self.mock_tracker = scaffold.MockTracker()
 
         self.test_fd = 274
 
@@ -424,8 +410,7 @@ class close_file_descriptor_if_open_TestCase(scaffold.TestCase):
             Called os.close(%(fd)r)
             """ % vars()
         daemon.daemon.close_file_descriptor_if_open(fd)
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_ignores_badfd_error_on_close(self):
         """ Should ignore OSError EBADF when closing """
@@ -438,42 +423,57 @@ class close_file_descriptor_if_open_TestCase(scaffold.TestCase):
             Called os.close(%(fd)r)
             """ % vars()
         daemon.daemon.close_file_descriptor_if_open(fd)
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
-    def test_propagates_error_on_close(self):
-        """ Should propagate OSError when closing """
+    def test_raises_error_if_error_on_close(self):
+        """ Should raise DaemonError if an OSError occurs when closing """
         fd = self.test_fd
         test_error = OSError(object(), "Unexpected error")
         def os_close(fd):
             raise test_error
         os.close.mock_returns_func = os_close
+        expect_error = daemon.daemon.DaemonOSEnvironmentError
         self.failUnlessRaises(
-            type(test_error),
+            expect_error,
             daemon.daemon.close_file_descriptor_if_open, fd)
 
 
 class maxfd_TestCase(scaffold.TestCase):
-    """ Test cases for module MAXFD """
+    """ Test cases for module MAXFD constant. """
 
     def test_positive(self):
-        """ Should be a positive number """
+        """ Should be a positive number. """
         maxfd = daemon.daemon.MAXFD
         self.failUnless(maxfd > 0)
 
     def test_integer(self):
-        """ Should be an integer """
+        """ Should be an integer. """
         maxfd = daemon.daemon.MAXFD
         self.failUnlessEqual(int(maxfd), maxfd)
 
+    def test_reasonably_high(self):
+        """ Should be reasonably high for default open files limit.
+
+            If the system reports a limit of “infinity” on maximum
+            file descriptors, we still need a finite number in order
+            to close “all” of them. Ensure this is reasonably high
+            to catch most use cases.
+
+            """
+        expect_minimum = 2048
+        maxfd = daemon.daemon.MAXFD
+        self.failUnless(
+            expect_minimum <= maxfd,
+            msg="MAXFD should be at least %(expect_minimum)r (got %(maxfd)r)"
+                % vars())
+
 
 class get_maximum_file_descriptors_TestCase(scaffold.TestCase):
     """ Test cases for get_maximum_file_descriptors function """
 
     def setUp(self):
         """ Set up test fixtures """
-        self.mock_outfile = StringIO()
-        self.mock_tracker = scaffold.MockTracker(self.mock_outfile)
+        self.mock_tracker = scaffold.MockTracker()
 
         self.RLIMIT_NOFILE = object()
         self.RLIM_INFINITY = object()
@@ -523,8 +523,7 @@ class close_all_open_files_TestCase(scaffold.TestCase):
 
     def setUp(self):
         """ Set up test fixtures """
-        self.mock_outfile = StringIO()
-        self.mock_tracker = scaffold.MockTracker(self.mock_outfile)
+        self.mock_tracker = scaffold.MockTracker()
 
         self.RLIMIT_NOFILE = object()
         self.RLIM_INFINITY = object()
@@ -568,8 +567,7 @@ class close_all_open_files_TestCase(scaffold.TestCase):
                 % vars()
             for fd in expect_file_descriptors)
         daemon.daemon.close_all_open_files()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_requests_all_but_excluded_files_to_close(self):
         """ Should request close of all open files but those excluded """
@@ -585,8 +583,7 @@ class close_all_open_files_TestCase(scaffold.TestCase):
                 % vars()
             for fd in expect_file_descriptors)
         daemon.daemon.close_all_open_files(**args)
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
 
 class detach_process_context_TestCase(scaffold.TestCase):
@@ -597,8 +594,7 @@ class detach_process_context_TestCase(scaffold.TestCase):
 
     def setUp(self):
         """ Set up test fixtures """
-        self.mock_outfile = StringIO()
-        self.mock_tracker = scaffold.MockTracker(self.mock_outfile)
+        self.mock_tracker = scaffold.MockTracker()
 
         test_pids = [0, 0]
         scaffold.mock(
@@ -631,9 +627,7 @@ class detach_process_context_TestCase(scaffold.TestCase):
         self.failUnlessRaises(
             self.FakeOSExit,
             daemon.daemon.detach_process_context)
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_first_fork_error_raises_error(self):
         """ Error on first fork should raise DaemonProcessDetachError """
@@ -657,9 +651,7 @@ class detach_process_context_TestCase(scaffold.TestCase):
         self.failUnlessRaises(
             daemon.daemon.DaemonProcessDetachError,
             daemon.daemon.detach_process_context)
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_child_starts_new_process_group(self):
         """ Child should start new process group """
@@ -669,9 +661,7 @@ class detach_process_context_TestCase(scaffold.TestCase):
             ...
             """
         daemon.daemon.detach_process_context()
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_child_forks_next_parent_exits(self):
         """ Child should fork, then exit if parent """
@@ -687,9 +677,7 @@ class detach_process_context_TestCase(scaffold.TestCase):
         self.failUnlessRaises(
             self.FakeOSExit,
             daemon.daemon.detach_process_context)
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_second_fork_error_reports_to_stderr(self):
         """ Error on second fork should cause report to stderr """
@@ -715,9 +703,7 @@ class detach_process_context_TestCase(scaffold.TestCase):
         self.failUnlessRaises(
             daemon.daemon.DaemonProcessDetachError,
             daemon.daemon.detach_process_context)
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_child_forks_next_child_continues(self):
         """ Child should fork, then continue if child """
@@ -727,9 +713,7 @@ class detach_process_context_TestCase(scaffold.TestCase):
             Called os.fork()
             """ % vars()
         daemon.daemon.detach_process_context()
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
 
 class is_process_started_by_init_TestCase(scaffold.TestCase):
@@ -737,8 +721,7 @@ class is_process_started_by_init_TestCase(scaffold.TestCase):
 
     def setUp(self):
         """ Set up test fixtures """
-        self.mock_outfile = StringIO()
-        self.mock_tracker = scaffold.MockTracker(self.mock_outfile)
+        self.mock_tracker = scaffold.MockTracker()
 
         self.test_ppid = 765
 
@@ -771,8 +754,7 @@ class is_socket_TestCase(scaffold.TestCase):
 
     def setUp(self):
         """ Set up test fixtures """
-        self.mock_outfile = StringIO()
-        self.mock_tracker = scaffold.MockTracker(self.mock_outfile)
+        self.mock_tracker = scaffold.MockTracker()
 
         def mock_getsockopt(level, optname, buflen=None):
             result = object()
@@ -836,8 +818,7 @@ class is_process_started_by_superserver_TestCase(scaffold.TestCase):
 
     def setUp(self):
         """ Set up test fixtures """
-        self.mock_outfile = StringIO()
-        self.mock_tracker = scaffold.MockTracker(self.mock_outfile)
+        self.mock_tracker = scaffold.MockTracker()
 
         def mock_is_socket(fd):
             if sys.__stdin__.fileno() == fd:
@@ -876,8 +857,7 @@ class is_detach_process_context_required_TestCase(scaffold.TestCase):
 
     def setUp(self):
         """ Set up test fixtures """
-        self.mock_outfile = StringIO()
-        self.mock_tracker = scaffold.MockTracker(self.mock_outfile)
+        self.mock_tracker = scaffold.MockTracker()
 
         scaffold.mock(
             "daemon.daemon.is_process_started_by_init",
@@ -913,9 +893,7 @@ class is_detach_process_context_required_TestCase(scaffold.TestCase):
 
 def setup_streams_fixtures(testcase):
     """ Set up common test fixtures for standard streams """
-    testcase.mock_outfile = StringIO()
-    testcase.mock_tracker = scaffold.MockTracker(
-        testcase.mock_outfile)
+    testcase.mock_tracker = scaffold.MockTracker()
 
     testcase.stream_file_paths = dict(
         stdin = tempfile.mktemp(),
@@ -976,8 +954,7 @@ class redirect_stream_TestCase(scaffold.TestCase):
             Called os.dup2(%(target_fileno)r, %(system_fileno)r)
             """ % vars()
         daemon.daemon.redirect_stream(system_stream, target_stream)
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_duplicates_null_file_descriptor_by_default(self):
         """ Should by default duplicate the null file to the system stream. """
@@ -993,8 +970,7 @@ class redirect_stream_TestCase(scaffold.TestCase):
             Called os.dup2(%(null_fileno)r, %(system_fileno)r)
             """ % vars()
         daemon.daemon.redirect_stream(system_stream, target_stream)
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
 
 class make_default_signal_map_TestCase(scaffold.TestCase):
@@ -1002,8 +978,7 @@ class make_default_signal_map_TestCase(scaffold.TestCase):
 
     def setUp(self):
         """ Set up test fixtures """
-        self.mock_outfile = StringIO()
-        self.mock_tracker = scaffold.MockTracker(self.mock_outfile)
+        self.mock_tracker = scaffold.MockTracker()
 
         default_signal_map_by_name = {
             'SIGCLD': None,
@@ -1061,8 +1036,7 @@ class set_signal_handlers_TestCase(scaffold.TestCase):
 
     def setUp(self):
         """ Set up test fixtures """
-        self.mock_outfile = StringIO()
-        self.mock_tracker = scaffold.MockTracker(self.mock_outfile)
+        self.mock_tracker = scaffold.MockTracker()
 
         scaffold.mock(
             "signal.signal",
@@ -1086,16 +1060,37 @@ class set_signal_handlers_TestCase(scaffold.TestCase):
                 % vars()
             for (signal_number, handler) in signal_handler_map.items())
         daemon.daemon.set_signal_handlers(signal_handler_map)
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
+
+
+class register_atexit_function_TestCase(scaffold.TestCase):
+    """ Test cases for register_atexit_function function. """
+
+    def setUp(self):
+        """ Set up test fixtures """
+        self.mock_tracker = scaffold.MockTracker()
+
+        scaffold.mock(
+            "atexit.register",
+            tracker=self.mock_tracker)
+
+    def tearDown(self):
+        """ Tear down test fixtures """
+        scaffold.mock_restore()
+
+    def test_registers_function_for_atexit_processing(self):
+        """ Should register specified function for atexit processing. """
+        func = object()
+        expect_mock_output = """\
+            Called atexit.register(%(func)r)
+            """ % vars()
+        daemon.daemon.register_atexit_function(func)
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
 
 def setup_daemon_context_fixtures(testcase):
     """ Set up common test fixtures for DaemonContext test case """
-
-    testcase.mock_outfile = StringIO()
-    testcase.mock_tracker = scaffold.MockTracker(
-        testcase.mock_outfile)
+    testcase.mock_tracker = scaffold.MockTracker()
 
     setup_streams_fixtures(testcase)
 
@@ -1313,6 +1308,7 @@ class DaemonContext_open_TestCase(scaffold.TestCase):
     def setUp(self):
         """ Set up test fixtures """
         setup_daemon_context_fixtures(self)
+        self.mock_tracker.clear()
 
         scaffold.mock(
             "daemon.daemon.detach_process_context",
@@ -1341,6 +1337,9 @@ class DaemonContext_open_TestCase(scaffold.TestCase):
         scaffold.mock(
             "daemon.daemon.set_signal_handlers",
             tracker=self.mock_tracker)
+        scaffold.mock(
+            "daemon.daemon.register_atexit_function",
+            tracker=self.mock_tracker)
 
         self.test_files_preserve_fds = object()
         scaffold.mock(
@@ -1389,11 +1388,11 @@ class DaemonContext_open_TestCase(scaffold.TestCase):
             Called daemon.daemon.redirect_stream(...)
             Called daemon.daemon.redirect_stream(...)
             Called pidlockfile.PIDLockFile.__enter__()
+            Called daemon.daemon.register_atexit_function(...)
             """ % vars()
-        self.mock_outfile.truncate(0)
+        self.mock_tracker.clear()
         instance.open()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_changes_root_directory_to_chroot_directory(self):
         """ Should change root directory to `chroot_directory` option. """
@@ -1401,38 +1400,31 @@ class DaemonContext_open_TestCase(scaffold.TestCase):
         chroot_directory = object()
         instance.chroot_directory = chroot_directory
         expect_mock_output = """\
-            ...
             Called daemon.daemon.change_root_directory(
                 %(chroot_directory)r)
             ...
             """ % vars()
         instance.open()
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_omits_chroot_if_no_chroot_directory(self):
         """ Should omit changing root directory if no `chroot_directory`. """
         instance = self.test_instance
         instance.chroot_directory = None
+        unwanted_output = """\
+            ...Called daemon.daemon.change_root_directory(...)..."""
         instance.open()
-        scaffold.mock_restore()
-        self.failIfIn(
-            self.mock_outfile.getvalue(),
-            "Called daemon.daemon.change_root_directory")
+        self.failIfMockCheckerMatch(unwanted_output)
 
     def test_prevents_core_dump(self):
         """ Should request prevention of core dumps """
         instance = self.test_instance
         expect_mock_output = """\
-            ...
             Called daemon.daemon.prevent_core_dump()
             ...
             """ % vars()
         instance.open()
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_closes_open_files(self):
         """ Should close all open files, excluding `files_preserve` """
@@ -1445,9 +1437,7 @@ class DaemonContext_open_TestCase(scaffold.TestCase):
             ...
             """ % vars()
         instance.open()
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_changes_directory_to_working_directory(self):
         """ Should change current directory to `working_directory` option. """
@@ -1461,9 +1451,7 @@ class DaemonContext_open_TestCase(scaffold.TestCase):
             ...
             """ % vars()
         instance.open()
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_changes_creation_mask_to_umask(self):
         """ Should change file creation mask to `umask` option. """
@@ -1476,9 +1464,7 @@ class DaemonContext_open_TestCase(scaffold.TestCase):
             ...
             """ % vars()
         instance.open()
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_changes_owner_to_specified_uid_and_gid(self):
         """ Should change process UID and GID to `uid` and `gid` options. """
@@ -1493,9 +1479,7 @@ class DaemonContext_open_TestCase(scaffold.TestCase):
             ...
             """ % vars()
         instance.open()
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_detaches_process_context(self):
         """ Should request detach of process context """
@@ -1506,19 +1490,16 @@ class DaemonContext_open_TestCase(scaffold.TestCase):
             ...
             """ % vars()
         instance.open()
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_omits_process_detach_if_not_required(self):
         """ Should omit detach of process context if not required """
         instance = self.test_instance
         instance.detach_process = False
+        unwanted_output = """\
+            ...Called daemon.daemon.detach_process_context(...)..."""
         instance.open()
-        scaffold.mock_restore()
-        self.failIfIn(
-            self.mock_outfile.getvalue(),
-            "Called daemon.daemon.detach_process_context")
+        self.failIfMockCheckerMatch(unwanted_output)
 
     def test_sets_signal_handlers_from_signal_map(self):
         """ Should set signal handlers according to `signal_map` """
@@ -1532,9 +1513,7 @@ class DaemonContext_open_TestCase(scaffold.TestCase):
             ...
             """ % vars()
         instance.open()
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_redirects_standard_streams(self):
         """ Should request redirection of standard stream files """
@@ -1552,11 +1531,10 @@ class DaemonContext_open_TestCase(scaffold.TestCase):
                 %(system_stdout)r, %(target_stdout)r)
             Called daemon.daemon.redirect_stream(
                 %(system_stderr)r, %(target_stderr)r)
+            ...
             """ % vars()
         instance.open()
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_enters_pidfile_context(self):
         """ Should enter the PID file context manager """
@@ -1565,30 +1543,54 @@ class DaemonContext_open_TestCase(scaffold.TestCase):
         expect_mock_output = """\
             ...
             Called pidlockfile.PIDLockFile.__enter__()
+            ...
             """
         instance.open()
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
+
+    def test_registers_close_method_for_atexit(self):
+        """ Should register the `close` method for atexit processing. """
+        instance = self.test_instance
+        close_method = instance.close
+        expect_mock_output = """\
+            ...
+            Called daemon.daemon.register_atexit_function(%(close_method)r)
+            """ % vars()
+        instance.open()
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
 
 class DaemonContext_close_TestCase(scaffold.TestCase):
-    """ Test cases for DaemonContext.close method """
+    """ Test cases for DaemonContext.close method. """
 
     def setUp(self):
-        """ Set up test fixtures """
+        """ Set up test fixtures. """
         setup_daemon_context_fixtures(self)
+        self.mock_tracker.clear()
 
     def tearDown(self):
-        """ Tear down test fixtures """
+        """ Tear down test fixtures. """
         scaffold.mock_restore()
 
+    def test_exits_pidfile_context(self):
+        """ Should exit the PID file context manager. """
+        instance = self.test_instance
+        instance.pidfile = self.mock_pidlockfile
+        expect_mock_output = """\
+            Called pidlockfile.PIDLockFile.__exit__()
+            """
+        expect_exception = SystemExit
+        try:
+            instance.close()
+        except expect_exception:
+            pass
+        self.failUnlessMockCheckerMatch(expect_mock_output)
+
     def test_returns_none(self):
         """ Should return None. """
         instance = self.test_instance
         expect_result = None
         result = instance.close()
-        scaffold.mock_restore()
         self.failUnlessIs(expect_result, result)
 
 
@@ -1598,6 +1600,7 @@ class DaemonContext_context_manager_enter_TestCase(scaffold.TestCase):
     def setUp(self):
         """ Set up test fixtures """
         setup_daemon_context_fixtures(self)
+        self.mock_tracker.clear()
 
         scaffold.mock(
             "daemon.daemon.DaemonContext.open",
@@ -1611,20 +1614,16 @@ class DaemonContext_context_manager_enter_TestCase(scaffold.TestCase):
         """ Should open the DaemonContext. """
         instance = self.test_instance
         expect_mock_output = """\
-            ...
             Called daemon.daemon.DaemonContext.open()
             """
         instance.__enter__()
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_returns_self_instance(self):
         """ Should return DaemonContext instance. """
         instance = self.test_instance
         expect_result = instance
         result = instance.__enter__()
-        scaffold.mock_restore()
         self.failUnlessIs(expect_result, result)
 
 
@@ -1634,6 +1633,7 @@ class DaemonContext_context_manager_exit_TestCase(scaffold.TestCase):
     def setUp(self):
         """ Set up test fixtures """
         setup_daemon_context_fixtures(self)
+        self.mock_tracker.clear()
 
         self.test_args = dict(
             exc_type = object(),
@@ -1654,13 +1654,10 @@ class DaemonContext_context_manager_exit_TestCase(scaffold.TestCase):
         instance = self.test_instance
         args = self.test_args
         expect_mock_output = """\
-            ...
             Called daemon.daemon.DaemonContext.close()
             """
         instance.__exit__(**args)
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
+        self.failUnlessMockCheckerMatch(expect_mock_output)
 
     def test_returns_none(self):
         """ Should return None, indicating exception was not handled. """
@@ -1668,7 +1665,6 @@ class DaemonContext_context_manager_exit_TestCase(scaffold.TestCase):
         args = self.test_args
         expect_result = None
         result = instance.__exit__(**args)
-        scaffold.mock_restore()
         self.failUnlessIs(expect_result, result)
 
 
@@ -1683,50 +1679,10 @@ class DaemonContext_terminate_TestCase(scaffold.TestCase):
         self.test_frame = None
         self.test_args = (self.test_signal, self.test_frame)
 
-        scaffold.mock(
-            "daemon.daemon.DaemonContext.close",
-            tracker=self.mock_tracker)
-
     def tearDown(self):
         """ Tear down test fixtures """
         scaffold.mock_restore()
 
-    def test_exits_pidfile_context(self):
-        """ Should exit the PID file context manager """
-        instance = self.test_instance
-        args = self.test_args
-        instance.pidfile = self.mock_pidlockfile
-        expect_mock_output = """\
-            ...
-            Called pidlockfile.PIDLockFile.__exit__()
-            ...
-            """
-        expect_exception = SystemExit
-        try:
-            instance.terminate(*args)
-        except expect_exception:
-            pass
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
-                          
-    def test_closes_daemon_context(self):
-        """ Should close the DaemonContext """
-        instance = self.test_instance
-        args = self.test_args
-        expect_mock_output = """\
-            ...
-            Called daemon.daemon.DaemonContext.close()
-            """
-        expect_exception = SystemExit
-        try:
-            instance.terminate(*args)
-        except expect_exception:
-            pass
-        scaffold.mock_restore()
-        self.failUnlessOutputCheckerMatch(
-            expect_mock_output, self.mock_outfile.getvalue())
-
     def test_raises_system_exit(self):
         """ Should raise SystemExit """
         instance = self.test_instance
@@ -1746,7 +1702,6 @@ class DaemonContext_terminate_TestCase(scaffold.TestCase):
             instance.terminate(*args)
         except expect_exception, exc:
             pass
-        scaffold.mock_restore()
         self.failUnlessIn(str(exc), str(signal_number))
 
 
